
  
        program hist
	use netcdf
	implicit none
        integer                :: i, j, k, is, t, ik, ist, bt, ew, ns
	parameter (ew = 180, ns = 40, bt = 40)
	real, dimension(bt)  :: model_z,vert_cld_frac_int,obs_z
	real                 :: h,time_dummy(25), toa_net_sw, toa_net_lw
	real, allocatable    :: FLW(:,:,:,:), FLWUP(:,:,:,:), FSW(:,:,:,:), FSWDN(:,:,:,:),  &
		              TAU_VIS(:,:,:,:), RRB(:,:,:,:), HEIGHT(:,:,:,:), sw_avg(:),  &
		              lw_avg(:), ht_avg(:)

	INTRINSIC :: TRIM, REAL, ABS
	
	open(10,file='ist',status="unknown")
	read(10,*) ist
	close (10)


!
!==========================================================================================
!			TOA_LONGWAVE, SHORTWAVE Radiation
!==========================================================================================
!

	allocate(FLW(ew,ns,bt,ist), FLWUP(ew,ns,bt,ist), FSW(ew,ns,bt,ist), FSWDN(ew,ns,bt,ist), &
	         TAU_VIS(ew,ns,bt,ist), RRB(ew,ns,bt,ist), HEIGHT(ew,ns,bt,ist), sw_avg(bt),     &
	         lw_avg(bt), ht_avg(bt))	

        	call read_wrf4d_output('FSW',3 , FSW)             ! NET Shortwave radiative flux (Downward positive)
        	call read_wrf4d_output('FLW',3 , FLW)             ! NET Longwave radiative flux (Upward positive) 
        	call read_wrf4d_output('FLWUP',5 , FLWUP)         ! Upward component of Longwave radiative flux
        	call read_wrf4d_output('FSWDN',5 , FSWDN)         ! Downward component of Shortwave radiative flux         	
        	call read_wrf4d_output('RRB',3 , RRB)
        	call read_wrf4d_output('HEIGHT',6 , HEIGHT)
        	call read_wrf4d_output('TAU_VIS',7 , TAU_VIS)
        	call read_wrf4d_output('FSW',3 , FSW)             ! NET Shortwave radiative flux (Upward positive)

          do k = 1, bt
                    sw_avg(k) = 0.; lw_avg(k) = 0.; ht_avg(k) = 0.; ik = 0;
                    do is = 1, ist
                              do i = 1, ew
                                        do j = 1, ns
                                                  sw_avg(k) = sw_avg(k)+FSWDN(i,j,k,is)
                                                  lw_avg(k) = lw_avg(k)+FLW(i,j,k,is)
                                                  ht_avg(k) = ht_avg(k)+HEIGHT(i,j,k,is)
                                                  ik = ik+1
                                        enddo
                              enddo
                    enddo
                    if(ik > 0)then
                              sw_avg(k) = sw_avg(k)/real(ik)
                              lw_avg(k) = lw_avg(k)/real(ik)
                              ht_avg(k) = ht_avg(k)/real(ik)
                    else
                              sw_avg(k) = -999.
                              lw_avg(k) = -999.
                              ht_avg(k) = -999. 
                    endif
          enddo
        	
          open(2,file='vertical_sw_lw_fluxes',status='unknown',action='write')
                    do k = 1, bt
                              write(2,*)ht_avg(k), sw_avg(k), lw_avg(k) 
                              write(*,*)ht_avg(k), sw_avg(k), lw_avg(k)
                    enddo
          close(2)
          
        CONTAINS
        SUBROUTINE read_wrf4d_output(var, nvar, WRFVAR)
                character  (len = *), parameter :: &
                        wrfout_file="wrfout_d01_2015-02-07_19:15:00"
                integer, intent(in) ::  nvar
                character *nvar :: var
                real , Dimension(ew,ns,bt,ist):: WRFVAR
                integer :: wrf_ncid, wrf_varid

                print*, wrfout_file//'_'//var//'.nc'
                ! Open the file. NF90_NOWRITE tells netCDF we want read-only access to
                ! the file.
                call check( nf90_open(wrfout_file//'_'//var//'.nc', NF90_NOWRITE, wrf_ncid) )

                print*, var
                ! Get the varid of the data variable, based on its name.
                call check( nf90_inq_varid(wrf_ncid, var, wrf_varid) )

                ! Read the data.
                print*, "reading::", var
                call check( nf90_get_var(wrf_ncid, wrf_varid, WRFVAR) )

        end subroutine read_wrf4d_output


        
       SUBROUTINE POLAT(XV, PV, X, P, plev)

	! INTERPOLATES XV TO PRESSURE P.

   	implicit none

   	integer, intent(in) :: plev

   	real, intent(in) ::&
     	p       !
   	real, dimension(plev), intent(in) ::&
      	XV,    &! DATA AT RESOLUTION N 
     	PV      ! PRESSURE AT N LEVELS
   	real, intent(out) ::&
      	X       ! DATA AT PRESSURE P

	! Local variables:

   	integer ::&
      	i, Ne, NM1
	!---------------------------------------------------------------------
   
   	Ne=plev
   	NM1=Ne-1

       IF (P .LE. PV(1)) THEN
		!       WRITE(6,2) P
          	X=(XV(2)-XV(1))/(PV(2)-PV(1))
         	X=X*(P-PV(1))+XV(1)
          	GO TO 1
       	ENDIF
      	IF (P .GE. PV(ne)) THEN
		!       WRITE(6,2) P
          	X=(XV(ne)-XV(nm1))/(PV(ne)-PV(nm1))
          	X=X*(P-PV(ne))+XV(ne)
          	GO TO 1
       	ENDIF
       	DO 10 I=1,NM1

       	IF ((P .LE. PV(i+1)) .AND. (P .GE. PV(I))) THEN
		!       WRITE(6,2) P
 		!2     FORMAT(2X,'POLATE P=',E4032.5)
		!       write(6,*) 'i,xv(i),xv(i+1)= ',i,xv(i),xv(i+1)
          	X=(XV(I+1)-XV(I))/(PV(I+1)-PV(I))
          	X=X*(P-PV(I+1))+XV(I+1)
          	GO TO 1
       	ENDIF

 	10    CONTINUE
 	1     CONTINUE

	END subroutine polat



        subroutine check(status)
                integer, intent ( in) :: status

                if(status /= nf90_noerr) then
                        print *, trim(nf90_strerror(status))
                        stop "Stopped"
                end if
        end subroutine check


	end
